@@ -26,6 +26,8 @@
 #include <gtk/gtkseparatormenuitem.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libgnome/gnome-help.h>
+#include <libgnomeui/gnome-ui-init.h>
 #include <eel/eel-vfs-extensions.h>
 #include <gconf/gconf-client.h>
 #include <time.h>
@@ -37,12 +39,17 @@
 #include "nautilus-cd-burner.h"
 #include "make-iso.h"
 
+enum {
+	CANCEL_NONE,
+	CANCEL_MAKE_ISO,
+	CANCEL_CD_RECORD,
+};
+
 static GladeXML *xml;
 
 static GList *recorders = NULL;
-static CancelFunc cancel_func = NULL;
-static gboolean cancel_dangerous = FALSE;
-static gpointer cancel_user_data;
+static CDRecorder *cdrecorder = NULL;
+static int cancel = CANCEL_NONE;
 /* For the image spinning */
 static GList *image_list;
 
@@ -72,16 +79,6 @@ cd_progress_set_text (const char *text)
 	gtk_label_set_text (GTK_LABEL (progress_label), text);
 }
 
-void
-cd_progress_set_cancel_func (CancelFunc _cancel_func,
-			     gboolean _cancel_dangerous,
-			     gpointer user_data)
-{
-	cancel_func = _cancel_func;
-	cancel_dangerous = _cancel_dangerous;
-	cancel_user_data = user_data;
-}
-
 static gboolean
 cd_progress_set_image (gpointer user_data)
 {
@@ -176,14 +173,6 @@ cd_progress_image_cleanup (void)
 	}
 }
 
-static void
-cancel_func_changed_cb (CDRecorder *cdrecorder, CancelFunc _cancel_func,
-			gboolean _cancel_dangerous, gpointer data)
-{
-	cd_progress_set_cancel_func (_cancel_func, _cancel_dangerous,
-				     (gpointer) cdrecorder);
-}
-
 static void
 animation_changed_cb (CDRecorder *cdrecorder, gboolean spinning, gpointer data)
 {
@@ -208,8 +197,6 @@ progress_changed_cb (CDRecorder *cdrecorder, gdouble fraction, gpointer data)
 static void
 reload_dialog_response (GtkDialog *dialog, gint response_id, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
-
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	cd_recorder_insert_cd_retry (cdrecorder, TRUE);
@@ -218,8 +205,6 @@ reload_dialog_response (GtkDialog *dialog, gint response_id, gpointer data)
 static void
 insert_cd_response (GtkDialog *dialog, gint response_id, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
-
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
@@ -228,8 +213,6 @@ insert_cd_response (GtkDialog *dialog, gint response_id, gpointer data)
 static void
 insert_cdrw_response (GtkDialog *dialog, gint response_id, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
-
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
@@ -311,6 +294,7 @@ media_type_matches (CDMediaType type, gboolean blank_cd)
 	switch (type)
 	{
 	case CD_MEDIA_TYPE_ERROR:
+	case CD_MEDIA_TYPE_BUSY:
 		return FALSE;
 	case CD_MEDIA_TYPE_UNKNOWN:
 		return TRUE;
@@ -397,7 +381,6 @@ wait_for_insertion (CDDrive *rec, gboolean blank_cd)
 static void
 details_clicked (GtkButton *button, gpointer data)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
 	GtkWidget *dialog, *label, *text, *scrolled_window;
 	GtkTextBuffer *buffer;
 
@@ -482,49 +465,6 @@ estimate_size_callback (const gchar *rel_path,
 	return TRUE;
 }
 
-gboolean
-is_audio (char *mime_type) 
-{
-	if (strcmp (mime_type, "audio/x-wav") == 0 || strcmp (mime_type, "audio/x-ulaw") == 0) {
-		return TRUE;
-	}
-	return FALSE;
-}
-
-static gboolean
-are_all_files_music (char *uri)
-{
-	GnomeVFSFileInfo *info;
-	GnomeVFSDirectoryHandle *handle;
-	GnomeVFSResult result;
-	gboolean all_music;
-
-	result = gnome_vfs_directory_open (&handle, uri, GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
-	if (result != GNOME_VFS_OK) {
-		return FALSE;
-	}
-	
-	info = gnome_vfs_file_info_new ();
-	all_music = TRUE;
-
-	while (TRUE) {
-		result = gnome_vfs_directory_read_next (handle, info);
-		if (result != GNOME_VFS_OK) {
-			break;
-		}
-		
-		if (!is_audio (info->mime_type)) {
-			all_music = FALSE;
-			break;
-		}
-	}
-
-	gnome_vfs_directory_close (handle);
-	gnome_vfs_file_info_unref (info);
-
-	return all_music;
-}
-
 static GnomeVFSFileSize
 estimate_size (char *uri)
 {
@@ -541,8 +481,10 @@ estimate_size (char *uri)
 		GList *list;
 
 		if (gnome_vfs_directory_list_load (&list, uri, GNOME_VFS_FILE_INFO_FIELDS_NONE) == GNOME_VFS_OK) {
-			size = 1;
-			gnome_vfs_file_info_list_free (list);
+			if (list != NULL) {
+				size = 1;
+				gnome_vfs_file_info_list_free (list);
+			}
 		}
 	}
 
@@ -562,22 +504,6 @@ setup_close_button (void)
 	g_signal_connect (close_button, "clicked", (GCallback)gtk_main_quit, NULL);
 }
 
-static void
-track_free (Track *track, gpointer user_data)
-{
-	switch (track->type) {
-	case TRACK_TYPE_DATA:
-		g_free (track->contents.data.filename);
-		break;
-	case TRACK_TYPE_AUDIO:
-		g_free (track->contents.audio.filename);
-		g_free (track->contents.audio.cdtext);
-		break;
-	default:
-		g_warning ("Invalid track type %d", track->type);
-	}
-}
-
 static gboolean 
 overwrite_existing_file (GtkWindow *filesel, const char *filename)
 {
@@ -668,10 +594,10 @@ file_chooser_response_cb (GtkWidget *chooser, gint response, gpointer data)
 		}
 
 		gtk_widget_destroy (GTK_WIDGET (chooser));
-		
+
 		g_free (*file_name_out);
 		*file_name_out = file_name;
-		
+
 		gtk_main_quit ();
 	}
 }
@@ -708,9 +634,8 @@ select_iso_filename (void)
 }
 
 static int
-burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, gboolean music_cd, int speed, gboolean dummy)
+burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, int speed, gboolean dummy)
 {
-	CDRecorder *cdrecorder;
 	GList *tracks;
 	char *filename, *str;
 	int fd;
@@ -720,12 +645,6 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 	filename = NULL;
 
 	if (rec->type == CDDRIVE_TYPE_FILE) {
-		if (source_iso != NULL) {
-			/* TODO: display some message here */
-			g_print ("Writing ISO onto ISO, you freak!\n");
-			return RESULT_FINISHED;
-		}
-
 		/* Run the file selector and get the iso file name selected */
 		filename = select_iso_filename ();
 
@@ -757,7 +676,7 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			}
 		}
 	} else {
-		if (source_iso == NULL && !music_cd) {
+		if (source_iso == NULL) {
 			GConfClient *gc;
 			const char *path;
 
@@ -769,12 +688,14 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				fd = mkstemp(filename);
 				close (fd);
 			}
-		} else if (!music_cd) {
+		} else {
 			filename = g_strdup (source_iso);
 		}
 	}
 
-	if (source_iso == NULL && !music_cd) {
+	if (source_iso == NULL) {
+		cancel = CANCEL_MAKE_ISO;
+
 		if (filename != NULL) {
 			res = make_iso (filename, label, FALSE, TRUE);
 		} else {
@@ -799,6 +720,7 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 
 			if (!res) {
 				/* User cancelled or we had an error. */
+				cancel = CANCEL_NONE;
 				goto out;
 			}
 		}
@@ -811,11 +733,14 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		setup_close_button ();
 		gtk_main ();
 	} else {
+		Track *track;
+
 		if (!wait_for_insertion (rec, blank_cd)) {
 			return RESULT_CANCEL;
 		}
 
 		cdrecorder = cd_recorder_new ();
+		cancel = CANCEL_CD_RECORD;
 
 		g_signal_connect (G_OBJECT (cdrecorder),
 				  "progress-changed",
@@ -829,10 +754,6 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "animation-changed",
 				  G_CALLBACK (animation_changed_cb),
 				  NULL);
-		g_signal_connect (G_OBJECT (cdrecorder),
-				  "cancel-func-changed",
-				  G_CALLBACK (cancel_func_changed_cb),
-				  NULL);
 		g_signal_connect (G_OBJECT (cdrecorder),
 				  "insert-cd-request",
 				  G_CALLBACK (insert_cd_request_cb),
@@ -841,18 +762,16 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "insert-rwcd-request",
 				  G_CALLBACK (insert_rwcd_request_cb),
 				  NULL);
-		
-		if (music_cd) {
-			tracks = build_list_of_audio_tracks ();
-		} else {
-			Track *track;
-			track = g_new0 (Track, 1);
-			track->type = TRACK_TYPE_DATA;
-			track->contents.data.filename = g_strdup (filename);
-			tracks = g_list_prepend (tracks, track);
-		}
+
+		track = g_new0 (Track, 1);
+		track->type = TRACK_TYPE_DATA;
+		track->contents.data.filename = g_strdup (filename);
+		tracks = g_list_prepend (tracks, track);
+
 		res = cd_recorder_write_tracks (cdrecorder, rec, tracks, speed, eject_cd, blank_cd, dummy, FALSE);
 
+		g_message ("filename: %s", filename);
+
 		if (source_iso == NULL) {
 			unlink (filename);
 		}
@@ -861,16 +780,17 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			cd_progress_set_text (_("Complete"));
 			setup_close_button ();
 			gtk_main ();
-		} else {
+		} else if (res != RESULT_CANCEL) {
 			show_error_message (cdrecorder);
 		}
-		
+
+		cancel = CANCEL_NONE;
 		g_object_unref (cdrecorder);
 	}
 	
  out:
 	g_free (filename);
-	g_list_foreach (tracks, (GFunc)track_free, NULL);
+	g_list_foreach (tracks, (GFunc)cd_recorder_track_free, NULL);
 	g_list_free (tracks);
 	return res;
 }
@@ -899,8 +819,6 @@ refresh_dialog (GladeXML *xml)
 	GtkWidget *item;
 	GtkWidget *eject_check;
 	GtkWidget *blank_check;
-	GtkWidget *music_check;
-	GtkWidget *name_entry;
 	GList *l, *children;
 	GConfClient *gc;
 	char *name;
@@ -950,16 +868,6 @@ refresh_dialog (GladeXML *xml)
 	/* Disable blank for burn-to-iso */
 	blank_check = glade_xml_get_widget (xml, "blank_check");
 	gtk_widget_set_sensitive (blank_check, rec->type & CDDRIVE_TYPE_CDRW_RECORDER);
-
-	/* Disable music for burn-to-iso */
-	music_check = glade_xml_get_widget (xml, "music_check");
-	gtk_widget_set_sensitive (music_check, rec->type != CDDRIVE_TYPE_FILE);
-
-	name_entry = glade_xml_get_widget (xml, "cdname_entry");
-	gtk_widget_set_sensitive (name_entry,
-				  rec->type == CDDRIVE_TYPE_FILE ||
-				  !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (music_check)));
-
 }
 
 static void
@@ -971,16 +879,7 @@ target_changed (GtkOptionMenu *option_menu, gpointer data)
 }
 
 static void
-music_cd_toggled (GtkToggleButton *button, gpointer data)
-{
-  GladeXML *xml = data;
-
-  refresh_dialog (xml);
-}
-
-
-static void
-init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean disable_reuse)
+init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean from_iso)
 {
 	GList *l;
 	CDDrive *rec;
@@ -990,7 +889,6 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *reuse;
-	GtkWidget *music_check;
 	int size_mb;
 	char *str;
 	char buf[129];
@@ -1024,11 +922,13 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	g_signal_connect (option_menu, "changed", (GCallback)target_changed, xml);
 	
 	/* Set default name */
-	
 	entry = glade_xml_get_widget (xml, "cdname_entry");
-	if (disc_name != NULL) {
-		gtk_entry_set_text (GTK_ENTRY (entry), disc_name);
-		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
+	if (from_iso != FALSE) {
+		gtk_widget_set_sensitive (entry, FALSE);
+		if (disc_name != NULL) {
+			gtk_entry_set_text (GTK_ENTRY (entry), disc_name);
+			gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
+		}
 	} else {
 		now = g_date_new ();
 
@@ -1046,44 +946,51 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 
 	reuse = glade_xml_get_widget (xml, "reuse_check");
 	g_assert (GTK_IS_WIDGET (reuse));
-	gtk_widget_set_sensitive (reuse, !disable_reuse);
+	gtk_widget_set_sensitive (reuse, !from_iso);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (reuse), TRUE);
 
-	music_check = glade_xml_get_widget (xml, "music_check");
-	if (!are_all_files_music ("burn:///")) {
-		gtk_widget_hide (music_check);	
-	}
-	g_signal_connect (music_check, "toggled", (GCallback)music_cd_toggled, xml);
-
-	
 	refresh_dialog (xml);
 }
 
+static gboolean
+ask_cancel (void)
+{
+	GtkWidget *warning_dialog;
+	int res;
+
+	warning_dialog = gtk_message_dialog_new (cd_progress_get_window (),
+						 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_WARNING,
+						 GTK_BUTTONS_YES_NO,
+						 _("Are you sure you want to cancel the CD write operation?\n"
+						 "Some CD writers may require that you restart the machine to get it working again."));
+	gtk_window_set_title (GTK_WINDOW (warning_dialog),
+			      _("Are you sure you want to cancel?"));
+	res = gtk_dialog_run (GTK_DIALOG (warning_dialog));
+	gtk_widget_destroy (warning_dialog);
+
+	return (res == GTK_RESPONSE_YES);
+}
+
 static void
 do_cancel (void)
 {
-	GtkWidget *warning_dialog;
-	gint res;
-	
-	if (cancel_dangerous) {
-		warning_dialog = gtk_message_dialog_new (cd_progress_get_window (),
-							 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
-							 GTK_MESSAGE_WARNING,
-							 GTK_BUTTONS_YES_NO,
-							 _("Are you sure you want to cancel the CD write operation?\n"
-							   "Some CD writers may require that you restart the machine to get it working again."));
-		gtk_window_set_title (GTK_WINDOW (warning_dialog),
-				      _("Are you sure you want to cancel?"));
-		res = gtk_dialog_run (GTK_DIALOG (warning_dialog));
-		gtk_widget_destroy (warning_dialog);
-		if (res != GTK_RESPONSE_YES) {
+	if (cancel == CANCEL_NONE) {
+		return;
+	}
+
+	if (cancel == CANCEL_MAKE_ISO) {
+		make_iso_cancel ();
+		return;
+	}
+
+	if (cd_recorder_cancel (cdrecorder, TRUE) != FALSE) {
+		if (ask_cancel () == FALSE) {
 			return;
 		}
 	}
 
-	if (cancel_func != NULL) {
-		(*cancel_func) (cancel_user_data);
-	}
+	cd_recorder_cancel (cdrecorder, FALSE);
 }
 
 static gboolean
@@ -1100,6 +1007,21 @@ cancel_clicked (GtkButton *button, gpointer data)
 	do_cancel ();
 }
 
+static void
+help_activate (GtkWindow *parent)
+{
+	GError *err = NULL;
+
+	if (gnome_help_display ("user-guide.xml", "gosnautilus-475", &err) == FALSE) {
+		char *msg;
+
+		msg = g_strdup_printf (_("There was a problem displaying the help contents: %s."), err->message);
+		display_error_dialog (parent, msg);
+		g_error_free (err);
+		g_free (msg);
+	}
+}
+
 static gboolean
 verify_iso (const char *filename, char **iso_label)
 {
@@ -1181,11 +1103,11 @@ int
 main (int argc, char *argv[])
 {
 	CDDrive *rec;
-	GtkWidget *dialog, *entry, *eject_check, *blank_check, *music_check, *reuse_check, *progress_window;
+	GtkWidget *dialog, *entry, *eject_check, *blank_check, *reuse_check, *progress_window;
 	GtkWidget *cancel_button, *dummy_check, *option_menu, *item;
 	const char *label;
 	int res, speed;
-	gboolean eject, blank, music, reuse, dummy;
+	gboolean eject, blank, reuse, dummy;
 	GnomeVFSURI *uri;
 	GnomeVFSFileSize size;
 	char *source_iso, *disc_name = NULL;
@@ -1196,10 +1118,12 @@ main (int argc, char *argv[])
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
+
+	gnome_program_init ("nautilus-cd-burner", VERSION,
+			    LIBGNOMEUI_MODULE, argc, argv,
+			    GNOME_PARAM_APP_DATADIR, SHAREDIR,
+			    GNOME_PARAM_NONE);
     
-	recorders = scan_for_cdroms (TRUE, TRUE);
-	
-	gtk_init (&argc, &argv);
 	gnome_vfs_init ();
 
 	if (argc > 2) {
@@ -1258,7 +1182,23 @@ main (int argc, char *argv[])
 			return 1;
 		}
 	}
-	
+
+	recorders = scan_for_cdroms (TRUE, source_iso != NULL ? FALSE : TRUE);
+
+	if (recorders == NULL) {
+		dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("You do not seem to have any CD recorder in your machine. Please verify your setup."));
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+		gtk_window_set_title (GTK_WINDOW (dialog),
+				      _("No CD recorder available"));
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		return 1;
+	}
+
 	xml = glade_xml_new (DATADIR "/cdburn.glade", NULL, NULL);
 	dialog = glade_xml_get_widget (xml, "cdr_dialog");
 
@@ -1277,7 +1217,13 @@ main (int argc, char *argv[])
 	icon = my_gdk_pixbuf_new_from_stock ("gtk-cdrom");
 	gtk_window_set_icon (GTK_WINDOW (dialog), icon);
 	gdk_pixbuf_unref (icon);
-	res = gtk_dialog_run (GTK_DIALOG (dialog));
+	do {
+		res = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		if (res == GTK_RESPONSE_HELP) {
+			help_activate (GTK_WINDOW (dialog));
+		}
+	} while (res == GTK_RESPONSE_HELP);
 
 	gtk_widget_hide (dialog);
 	progress_window = glade_xml_get_widget (xml, "progress_window");
@@ -1289,7 +1235,7 @@ main (int argc, char *argv[])
 
 	if (res == 0) {
 		rec = lookup_current_recorder (xml);
-		
+
 		entry = glade_xml_get_widget (xml, "cdname_entry");
 		label = gtk_entry_get_text (GTK_ENTRY (entry));
 
@@ -1299,10 +1245,6 @@ main (int argc, char *argv[])
 		blank_check = glade_xml_get_widget (xml, "blank_check");
 		blank = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (blank_check));
 
-		music_check = glade_xml_get_widget (xml, "music_check");
-		music = (rec->type != CDDRIVE_TYPE_FILE) &&
-			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (music_check));
-	
 		dummy = FALSE;
 
 #ifdef ENABLE_DUMMY_WRITE
@@ -1316,7 +1258,7 @@ main (int argc, char *argv[])
 		gconf_client_set_int (gc, "/apps/nautilus-cd-burner/default_speed", speed, NULL);
 		g_object_unref (G_OBJECT (gc));
 
-		res = burn_cd (rec, source_iso, label, eject, blank, music, speed, dummy);
+		res = burn_cd (rec, source_iso, label, eject, blank, speed, dummy);
 
 		reuse_check = glade_xml_get_widget (xml, "reuse_check");
 		reuse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reuse_check));
